@@ -380,8 +380,8 @@ cdfs_show(struct seq_file *file_p, void *data_p) {
  ******************************************************************************/
 
 struct seq_operations cdfs_operations = {
-    start:	cdfs_start,
-    next:	cdfs_next,
-    stop:	cdfs_stop,
-    show:	cdfs_show
+    .start = cdfs_start,
+    .next  = cdfs_next,
+    .stop  = cdfs_stop,
+    .show  = cdfs_show,
 };
